@@ -45,6 +45,22 @@ _SERVER_FIELDS = (
     'user_id',
 )
 
+_COMPUTE_LIMITS_FIELDS = (
+    ('maxPersonality', 'max_personality'),
+    ('maxPersonalitySize', 'max_personality_size'),
+    ('maxServerGroupMembers', 'max_server_group_members'),
+    ('maxServerGroups', 'max_server_groups'),
+    ('maxServerMeta', 'max_server_meta'),
+    ('maxTotalCores', 'max_total_cores'),
+    ('maxTotalInstances', 'max_total_instances'),
+    ('maxTotalKeypairs', 'max_total_keypairs'),
+    ('maxTotalRAMSize', 'max_total_ram_size'),
+    ('totalCoresUsed', 'total_cores_used'),
+    ('totalInstancesUsed', 'total_instances_used'),
+    ('totalRAMUsed', 'total_ram_used'),
+    ('totalServerGroupsUsed', 'total_server_groups_used'),
+)
+
 
 _pushdown_fields = {
     'project': [
@@ -106,6 +122,26 @@ class Normalizer(object):
     reasons.
     '''
 
+    def _normalize_compute_limits(self, limits, project_id=None):
+        """ Normalize a limits object.
+
+        Limits modified in this method and shouldn't be modified afterwards.
+        """
+
+        # Copy incoming limits because of shared dicts in unittests
+        limits = limits['absolute'].copy()
+
+        new_limits = munch.Munch()
+        new_limits['location'] = self._get_current_location(
+            project_id=project_id)
+
+        for field in _COMPUTE_LIMITS_FIELDS:
+            new_limits[field[1]] = limits.pop(field[0], None)
+
+        new_limits['properties'] = limits.copy()
+
+        return new_limits
+
     def _normalize_flavors(self, flavors):
         """ Normalize a list of flavor objects """
         ret = []
@@ -578,6 +614,56 @@ class Normalizer(object):
 
         return ret
 
+    def _normalize_volume_type_access(self, volume_type_access):
+
+        volume_type_access = volume_type_access.copy()
+
+        volume_type_id = volume_type_access.pop('volume_type_id')
+        project_id = volume_type_access.pop('project_id')
+        ret = munch.Munch(
+            location=self.current_location,
+            project_id=project_id,
+            volume_type_id=volume_type_id,
+            properties=volume_type_access.copy(),
+        )
+        return ret
+
+    def _normalize_volume_type_accesses(self, volume_type_accesses):
+        ret = []
+        for volume_type_access in volume_type_accesses:
+            ret.append(self._normalize_volume_type_access(volume_type_access))
+        return ret
+
+    def _normalize_volume_type(self, volume_type):
+
+        volume_type = volume_type.copy()
+
+        volume_id = volume_type.pop('id')
+        description = volume_type.pop('description', None)
+        name = volume_type.pop('name', None)
+        old_is_public = volume_type.pop('os-volume-type-access:is_public',
+                                        False)
+        is_public = volume_type.pop('is_public', old_is_public)
+        qos_specs_id = volume_type.pop('qos_specs_id', None)
+        extra_specs = volume_type.pop('extra_specs', {})
+        ret = munch.Munch(
+            location=self.current_location,
+            is_public=is_public,
+            id=volume_id,
+            name=name,
+            description=description,
+            qos_specs_id=qos_specs_id,
+            extra_specs=extra_specs,
+            properties=volume_type.copy(),
+        )
+        return ret
+
+    def _normalize_volume_types(self, volume_types):
+        ret = []
+        for volume in volume_types:
+            ret.append(self._normalize_volume_type(volume))
+        return ret
+
     def _normalize_volumes(self, volumes):
         """Normalize the structure of volumes
 
@@ -673,3 +759,15 @@ class Normalizer(object):
             for key, val in ret['properties'].items():
                 ret.setdefault(key, val)
         return ret
+
+    def _normalize_usage(self, usage):
+        """ Normalize a usage object """
+
+        # Discard noise
+        usage.pop('links', None)
+        usage.pop('NAME_ATTR', None)
+        usage.pop('HUMAN_ID', None)
+        usage.pop('human_id', None)
+        usage.pop('request_ids', None)
+
+        return munch.Munch(usage)
