@@ -102,6 +102,11 @@ class FlavorGet(task_manager.Task):
         return client.nova_client.flavors.get(**self.args)
 
 
+class FlavorListAccess(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.flavor_access.list(**self.args)
+
+
 class FlavorAddAccess(task_manager.Task):
     def main(self, client):
         return client.nova_client.flavor_access.add_tenant_access(
@@ -296,6 +301,28 @@ class ImageSnapshotCreate(task_manager.Task):
         return client.nova_client.servers.create_image(**self.args)
 
 
+class VolumeTypeList(task_manager.Task):
+    def main(self, client):
+        return client.cinder_client.volume_types.list()
+
+
+class VolumeTypeAccessList(task_manager.Task):
+    def main(self, client):
+        return client.cinder_client.volume_type_access.list(**self.args)
+
+
+class VolumeTypeAccessAdd(task_manager.Task):
+    def main(self, client):
+        return client.cinder_client.volume_type_access.add_project_access(
+            **self.args)
+
+
+class VolumeTypeAccessRemove(task_manager.Task):
+    def main(self, client):
+        return client.cinder_client.volume_type_access.remove_project_access(
+            **self.args)
+
+
 class VolumeCreate(task_manager.Task):
     def main(self, client):
         return client.cinder_client.volumes.create(**self.args)
@@ -466,11 +493,6 @@ class FloatingIPPoolList(task_manager.Task):
         return client.nova_client.floating_ip_pools.list()
 
 
-class ObjectCreate(task_manager.Task):
-    def main(self, client):
-        return client.swift_service.upload(**self.args)
-
-
 class SubnetCreate(task_manager.Task):
     def main(self, client):
         return client.neutron_client.create_subnet(**self.args)
@@ -806,6 +828,11 @@ class NovaQuotasDelete(task_manager.Task):
         return client.nova_client.quotas.delete(**self.args)
 
 
+class NovaUsageGet(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.usage.get(**self.args)
+
+
 class CinderQuotasSet(task_manager.Task):
     def main(self, client):
         return client.cinder_client.quotas.update(**self.args)
@@ -860,3 +887,8 @@ class ClusterTemplateUpdate(task_manager.Task):
 class MagnumServicesList(task_manager.Task):
     def main(self, client):
         return client.magnum_client.mservices.list(detail=False)
+
+
+class NovaLimitsGet(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.limits.get(**self.args).to_dict()
