@@ -56,6 +56,9 @@ def _iterate_timeout(timeout, message, wait=2):
         # seems friendlier
         if wait is None:
             wait = 2
+        elif wait == 0:
+            # wait should be < timeout, unless timeout is None
+            wait = 0.1 if timeout is None else min(0.1, timeout)
         wait = float(wait)
     except ValueError:
         raise exc.OpenStackCloudException(
@@ -303,6 +306,17 @@ def normalize_roles(roles):
     return meta.obj_list_to_dict(ret)
 
 
+def normalize_flavor_accesses(flavor_accesses):
+    """Normalize Flavor access list."""
+    return [munch.Munch(
+        dict(
+            flavor_id=acl.get('flavor_id'),
+            project_id=acl.get('project_id') or acl.get('tenant_id'),
+        )
+    ) for acl in flavor_accesses
+    ]
+
+
 def normalize_stacks(stacks):
     """ Normalize Stack Object """
     for stack in stacks:
@@ -602,3 +616,40 @@ def generate_patches_from_kwargs(operation, **kwargs):
                  'path': '/%s' % k}
         patches.append(patch)
     return patches
+
+
+class FileSegment(object):
+    """File-like object to pass to requests."""
+
+    def __init__(self, filename, offset, length):
+        self.filename = filename
+        self.offset = offset
+        self.length = length
+        self.pos = 0
+        self._file = open(filename, 'rb')
+        self.seek(0)
+
+    def tell(self):
+        return self._file.tell() - self.offset
+
+    def seek(self, offset, whence=0):
+        if whence == 0:
+            self._file.seek(self.offset + offset, whence)
+        elif whence == 1:
+            self._file.seek(offset, whence)
+        elif whence == 2:
+            self._file.seek(self.offset + self.length - offset, 0)
+
+    def read(self, size=-1):
+        remaining = self.length - self.pos
+        if remaining <= 0:
+            return b''
+
+        to_read = remaining if size < 0 else min(size, remaining)
+        chunk = self._file.read(to_read)
+        self.pos += len(chunk)
+
+        return chunk
+
+    def reset(self):
+        self._file.seek(self.offset, 0)
