@@ -1593,6 +1593,23 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         """
         self._mod_flavor_access('remove', flavor_id, project_id)
 
+    def list_flavor_access(self, flavor_id):
+        """List access from a private flavor for a project/tenant.
+
+        :param string flavor_id: ID of the private flavor.
+
+        :returns: a list of ``munch.Munch`` containing the access description
+
+        :raises: OpenStackCloudException on operation error.
+        """
+        with _utils.shade_exceptions("Error trying to list access from "
+                                     "flavor ID {flavor}".format(
+                flavor=flavor_id)):
+            projects = self.manager.submit_task(
+                _tasks.FlavorListAccess(flavor=flavor_id)
+            )
+        return _utils.normalize_flavor_accesses(projects)
+
     def create_role(self, name):
         """Create a Keystone role.
 
@@ -1960,6 +1977,70 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
             return self.manager.submit_task(_tasks.AggregateRemoveHost(
                 aggregate=aggregate['id'], host=host_name))
 
+    def get_volume_type_access(self, name_or_id):
+        """Return a list of volume_type_access.
+
+        :param name_or_id: Name or ID of the volume type.
+
+        :raises: OpenStackCloudException on operation error.
+        """
+        volume_type = self.get_volume_type(name_or_id)
+        if not volume_type:
+            raise OpenStackCloudException(
+                "VolumeType not found: %s" % name_or_id)
+
+        with _utils.shade_exceptions(
+                "Unable to get volume type access {name}".format(
+                    name=name_or_id)):
+            return self._normalize_volume_type_accesses(
+                self.manager.submit_task(
+                    _tasks.VolumeTypeAccessList(volume_type=volume_type))
+            )
+
+    def add_volume_type_access(self, name_or_id, project_id):
+        """Grant access on a volume_type to a project.
+
+        :param name_or_id: ID or name of a volume_type
+        :param project_id: A project id
+
+        NOTE: the call works even if the project does not exist.
+
+        :raises: OpenStackCloudException on operation error.
+        """
+        volume_type = self.get_volume_type(name_or_id)
+        if not volume_type:
+            raise OpenStackCloudException(
+                "VolumeType not found: %s" % name_or_id)
+        with _utils.shade_exceptions(
+                "Unable to authorize {project} "
+                "to use volume type {name}".format(
+                    name=name_or_id, project=project_id
+                )):
+            self.manager.submit_task(
+                _tasks.VolumeTypeAccessAdd(
+                    volume_type=volume_type, project=project_id))
+
+    def remove_volume_type_access(self, name_or_id, project_id):
+        """Revoke access on a volume_type to a project.
+
+        :param name_or_id: ID or name of a volume_type
+        :param project_id: A project id
+
+        :raises: OpenStackCloudException on operation error.
+        """
+        volume_type = self.get_volume_type(name_or_id)
+        if not volume_type:
+            raise OpenStackCloudException(
+                "VolumeType not found: %s" % name_or_id)
+        with _utils.shade_exceptions(
+                "Unable to revoke {project} "
+                "to use volume type {name}".format(
+                    name=name_or_id, project=project_id
+                )):
+            self.manager.submit_task(
+                _tasks.VolumeTypeAccessRemove(
+                    volume_type=volume_type, project=project_id))
+
     def set_compute_quotas(self, name_or_id, **kwargs):
         """ Set a quota in a project
 
@@ -2025,6 +2106,29 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         except nova_exceptions.BadRequest:
             raise OpenStackCloudException("nova client call failed")
 
+    def get_compute_usage(self, name_or_id, start, end):
+        """ Get usage for a specific project
+
+        :param name_or_id: project name or id
+        :param start: :class:`datetime.datetime` Start date in UTC
+        :param end: :class:`datetime.datetime` End date in UTCs
+        :raises: OpenStackCloudException if it's not a valid project
+
+        :returns: Munch object with the usage
+        """
+        proj = self.get_project(name_or_id)
+        if not proj:
+            raise OpenStackCloudException("project does not exist: {}".format(
+                name=proj.id))
+
+        with _utils.shade_exceptions(
+                "Unable to get resources usage for project: {name}".format(
+                    name=proj.id)):
+            usage = self.manager.submit_task(
+                _tasks.NovaUsageGet(tenant_id=proj.id, start=start, end=end))
+
+        return self._normalize_usage(usage)
+
     def set_volume_quotas(self, name_or_id, **kwargs):
         """ Set a volume quota in a project
 
