@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # Copyright 2010-2011 OpenStack Foundation
 # Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
 #
@@ -21,6 +19,7 @@ import fixtures
 import mock
 import os
 import os_client_config as occ
+from requests import structures
 from requests_mock.contrib import fixture as rm_fixture
 import tempfile
 
@@ -113,17 +112,20 @@ class RequestsMockTestCase(BaseTestCase):
         super(RequestsMockTestCase, self).setUp(
             cloud_config_fixture=cloud_config_fixture)
 
+        self._uri_registry = {}
+
         self.discovery_json = os.path.join(
             self.fixtures_directory, 'discovery.json')
         self.use_keystone_v3()
 
     def use_keystone_v3(self):
         self.adapter = self.useFixture(rm_fixture.Fixture())
-        self.adapter.get(
-            'http://192.168.0.19:35357/',
+        self.calls = []
+        self.register_uri(
+            'GET', 'http://192.168.0.19:35357/',
             text=open(self.discovery_json, 'r').read())
-        self.adapter.post(
-            'https://example.com/v3/auth/tokens',
+        self.register_uri(
+            'POST', 'https://example.com/v3/auth/tokens',
             headers={
                 'X-Subject-Token': self.getUniqueString()},
             text=open(
@@ -131,28 +133,21 @@ class RequestsMockTestCase(BaseTestCase):
                     self.fixtures_directory,
                     'catalog-v3.json'),
                 'r').read())
-        self.calls = [
-            dict(method='GET', url='http://192.168.0.19:35357/'),
-            dict(method='POST', url='https://example.com/v3/auth/tokens'),
-        ]
         self._make_test_cloud(identity_api_version='3')
 
     def use_keystone_v2(self):
         self.adapter = self.useFixture(rm_fixture.Fixture())
-        self.adapter.get(
-            'http://192.168.0.19:35357/',
+        self.calls = []
+        self.register_uri(
+            'GET', 'http://192.168.0.19:35357/',
             text=open(self.discovery_json, 'r').read())
-        self.adapter.post(
-            'https://example.com/v2.0/tokens',
+        self.register_uri(
+            'POST', 'https://example.com/v2.0/tokens',
             text=open(
                 os.path.join(
                     self.fixtures_directory,
                     'catalog-v2.json'),
                 'r').read())
-        self.calls = [
-            dict(method='GET', url='http://192.168.0.19:35357/'),
-            dict(method='POST', url='https://example.com/v2.0/tokens'),
-        ]
         self._make_test_cloud(identity_api_version='2.0')
 
     def _make_test_cloud(self, **kwargs):
@@ -166,17 +161,37 @@ class RequestsMockTestCase(BaseTestCase):
     def use_glance(self, image_version_json='image-version.json'):
         discovery_fixture = os.path.join(
             self.fixtures_directory, image_version_json)
-        self.adapter.get(
-            'https://image.example.com/',
+        self.register_uri(
+            'GET', 'https://image.example.com/',
             text=open(discovery_fixture, 'r').read())
+
+    def register_uri(self, method, uri, **kwargs):
+        validate = kwargs.pop('validate', {})
+        key = '{method}:{uri}'.format(method=method, uri=uri)
+        headers = structures.CaseInsensitiveDict(kwargs.pop('headers', {}))
+        if 'content-type' not in headers:
+            headers[u'content-type'] = 'application/json'
+        kwargs['headers'] = headers
+
+        if key in self._uri_registry:
+            self._uri_registry[key].append(kwargs)
+            self.adapter.register_uri(method, uri, self._uri_registry[key])
+        else:
+            self._uri_registry[key] = [kwargs]
+            self.adapter.register_uri(method, uri, **kwargs)
+
         self.calls += [
-            dict(method='GET', url='https://image.example.com/'),
+            dict(
+                method=method,
+                url=uri, **validate)
         ]
 
-    def assert_calls(self):
+    def assert_calls(self, stop_after=None):
         self.assertEqual(len(self.calls), len(self.adapter.request_history))
         for (x, (call, history)) in enumerate(
                 zip(self.calls, self.adapter.request_history)):
+            if stop_after and x > stop_after:
+                break
             self.assertEqual(
                 call['method'], history.method,
                 'Method mismatch on call {index}'.format(index=x))
