@@ -31,10 +31,10 @@ NO_MD5 = '93b885adfe0da089cdf634904fd59f71'
 NO_SHA256 = '6e340b9cffb37a989ca544e6bb780a2c78901d3fb33738768511a30617afa01d'
 
 
-class TestImage(base.RequestsMockTestCase):
+class BaseTestImage(base.RequestsMockTestCase):
 
     def setUp(self):
-        super(TestImage, self).setUp()
+        super(BaseTestImage, self).setUp()
         self.image_id = str(uuid.uuid4())
         self.imagefile = tempfile.NamedTemporaryFile(delete=False)
         self.imagefile.write(b'\0')
@@ -68,6 +68,12 @@ class TestImage(base.RequestsMockTestCase):
             u'protected': False}
         self.fake_search_return = {'images': [self.fake_image_dict]}
         self.output = uuid.uuid4().bytes
+
+
+class TestImage(BaseTestImage):
+
+    def setUp(self):
+        super(TestImage, self).setUp()
         self.use_glance()
 
     def test_config_v1(self):
@@ -101,19 +107,20 @@ class TestImage(base.RequestsMockTestCase):
                           output_path='fake_path', output_file=fake_fd)
 
     def test_download_image_no_images_found(self):
-        self.adapter.get(
-            'https://image.example.com/v2/images',
+        self.register_uri(
+            'GET', 'https://image.example.com/v2/images',
             json=dict(images=[]))
         self.assertRaises(exc.OpenStackCloudResourceNotFound,
                           self.cloud.download_image, 'fake_image',
                           output_path='fake_path')
+        self.assert_calls()
 
     def _register_image_mocks(self):
-        self.adapter.get(
-            'https://image.example.com/v2/images',
+        self.register_uri(
+            'GET', 'https://image.example.com/v2/images',
             json=self.fake_search_return)
-        self.adapter.get(
-            'https://image.example.com/v2/images/{id}/file'.format(
+        self.register_uri(
+            'GET', 'https://image.example.com/v2/images/{id}/file'.format(
                 id=self.image_id),
             content=self.output,
             headers={
@@ -126,6 +133,7 @@ class TestImage(base.RequestsMockTestCase):
         self.cloud.download_image('fake_image', output_file=output_file)
         output_file.seek(0)
         self.assertEqual(output_file.read(), self.output)
+        self.assert_calls()
 
     def test_download_image_with_path(self):
         self._register_image_mocks()
@@ -133,29 +141,32 @@ class TestImage(base.RequestsMockTestCase):
         self.cloud.download_image('fake_image', output_path=output_file.name)
         output_file.seek(0)
         self.assertEqual(output_file.read(), self.output)
+        self.assert_calls()
 
     def test_empty_list_images(self):
-        self.adapter.register_uri(
+        self.register_uri(
             'GET', 'https://image.example.com/v2/images', json={'images': []})
         self.assertEqual([], self.cloud.list_images())
+        self.assert_calls()
 
     def test_list_images(self):
-        self.adapter.register_uri(
+        self.register_uri(
             'GET', 'https://image.example.com/v2/images',
             json=self.fake_search_return)
         self.assertEqual(
             self.cloud._normalize_images([self.fake_image_dict]),
             self.cloud.list_images())
+        self.assert_calls()
 
     def test_list_images_paginated(self):
         marker = str(uuid.uuid4())
-        self.adapter.register_uri(
+        self.register_uri(
             'GET', 'https://image.example.com/v2/images',
             json={
                 'images': [self.fake_image_dict],
                 'next': '/v2/images?marker={marker}'.format(marker=marker),
             })
-        self.adapter.register_uri(
+        self.register_uri(
             'GET',
             'https://image.example.com/v2/images?marker={marker}'.format(
                 marker=marker),
@@ -164,33 +175,19 @@ class TestImage(base.RequestsMockTestCase):
             self.cloud._normalize_images([
                 self.fake_image_dict, self.fake_image_dict]),
             self.cloud.list_images())
+        self.assert_calls()
 
     def test_create_image_put_v2(self):
         self.cloud.image_api_use_tasks = False
 
-        self.adapter.register_uri(
-            'GET', 'https://image.example.com/v2/images', [
-                dict(json={'images': []}),
-                dict(json=self.fake_search_return),
-            ])
-        self.adapter.register_uri(
+        self.register_uri(
+            'GET', 'https://image.example.com/v2/images',
+            json={'images': []})
+
+        self.register_uri(
             'POST', 'https://image.example.com/v2/images',
             json=self.fake_image_dict,
-        )
-        self.adapter.register_uri(
-            'PUT', 'https://image.example.com/v2/images/{id}/file'.format(
-                id=self.image_id),
-            request_headers={'Content-Type': 'application/octet-stream'})
-
-        self.cloud.create_image(
-            'fake_image', self.imagefile.name, wait=True, timeout=1,
-            is_public=False)
-
-        self.calls += [
-            dict(method='GET', url='https://image.example.com/v2/images'),
-            dict(
-                method='POST',
-                url='https://image.example.com/v2/images',
+            validate=dict(
                 json={
                     u'container_format': u'bare',
                     u'disk_format': u'qcow2',
@@ -200,31 +197,46 @@ class TestImage(base.RequestsMockTestCase):
                     u'owner_specified.shade.sha256': NO_SHA256,
                     u'visibility': u'private'
                 }),
-            dict(
-                method='PUT',
-                url='https://image.example.com/v2/images/{id}/file'.format(
-                    id=self.image_id)),
-            dict(method='GET', url='https://image.example.com/v2/images'),
-        ]
+        )
+        self.register_uri(
+            'PUT', 'https://image.example.com/v2/images/{id}/file'.format(
+                id=self.image_id),
+            request_headers={'Content-Type': 'application/octet-stream'})
+
+        self.register_uri(
+            'GET', 'https://image.example.com/v2/images',
+            json=self.fake_search_return)
+
+        self.cloud.create_image(
+            'fake_image', self.imagefile.name, wait=True, timeout=1,
+            is_public=False)
+
         self.assert_calls()
         self.assertEqual(self.adapter.request_history[5].text.read(), b'\x00')
 
-    @mock.patch.object(shade.OpenStackCloud, 'swift_service')
-    def test_create_image_task(self,
-                               swift_service_mock):
+    def test_create_image_task(self):
         self.cloud.image_api_use_tasks = True
         image_name = 'name-99'
         container_name = 'image_upload_v2_test_container'
         endpoint = self.cloud._object_store_client.get_endpoint()
 
-        self.adapter.get(
-            'https://object-store.example.com/info',
+        self.register_uri(
+            'GET', 'https://image.example.com/v2/images', json={'images': []})
+
+        self.register_uri(
+            'GET', 'https://object-store.example.com/info',
             json=dict(
                 swift={'max_file_size': 1000},
                 slo={'min_segment_size': 500}))
 
-        self.adapter.put(
-            '{endpoint}/{container}'.format(
+        self.register_uri(
+            'HEAD', '{endpoint}/{container}'.format(
+                endpoint=endpoint,
+                container=container_name),
+            status_code=404)
+
+        self.register_uri(
+            'PUT', '{endpoint}/{container}'.format(
                 endpoint=endpoint,
                 container=container_name,),
             status_code=201,
@@ -233,29 +245,39 @@ class TestImage(base.RequestsMockTestCase):
                 'Content-Length': '0',
                 'Content-Type': 'text/html; charset=UTF-8',
             })
-        self.adapter.head(
-            '{endpoint}/{container}'.format(
+
+        self.register_uri(
+            'HEAD', '{endpoint}/{container}'.format(
                 endpoint=endpoint,
                 container=container_name),
-            [
-                dict(status_code=404),
-                dict(headers={
-                    'Content-Length': '0',
-                    'X-Container-Object-Count': '0',
-                    'Accept-Ranges': 'bytes',
-                    'X-Storage-Policy': 'Policy-0',
-                    'Date': 'Fri, 16 Dec 2016 18:29:05 GMT',
-                    'X-Timestamp': '1481912480.41664',
-                    'X-Trans-Id': 'tx60ec128d9dbf44b9add68-0058543271dfw1',
-                    'X-Container-Bytes-Used': '0',
-                    'Content-Type': 'text/plain; charset=utf-8'}),
-            ])
-        self.adapter.head(
-            '{endpoint}/{container}/{object}'.format(
+            headers={
+                'Content-Length': '0',
+                'X-Container-Object-Count': '0',
+                'Accept-Ranges': 'bytes',
+                'X-Storage-Policy': 'Policy-0',
+                'Date': 'Fri, 16 Dec 2016 18:29:05 GMT',
+                'X-Timestamp': '1481912480.41664',
+                'X-Trans-Id': 'tx60ec128d9dbf44b9add68-0058543271dfw1',
+                'X-Container-Bytes-Used': '0',
+                'Content-Type': 'text/plain; charset=utf-8'})
+
+        self.register_uri(
+            'HEAD', '{endpoint}/{container}/{object}'.format(
                 endpoint=endpoint,
                 container=container_name, object=image_name),
             status_code=404)
 
+        self.register_uri(
+            'PUT', '{endpoint}/{container}/{object}'.format(
+                endpoint=endpoint,
+                container=container_name, object=image_name),
+            status_code=201,
+            validate=dict(
+                headers={
+                    'x-object-meta-x-shade-md5': NO_MD5,
+                    'x-object-meta-x-shade-sha256': NO_SHA256,
+                }))
+
         task_id = str(uuid.uuid4())
         args = dict(
             id=task_id,
@@ -271,90 +293,39 @@ class TestImage(base.RequestsMockTestCase):
         del(image_no_checksums['owner_specified.shade.sha256'])
         del(image_no_checksums['owner_specified.shade.object'])
 
-        self.adapter.register_uri(
-            'GET', 'https://image.example.com/v2/images', [
-                dict(json={'images': []}),
-                dict(json={'images': []}),
-                dict(json={'images': [image_no_checksums]}),
-                dict(json=self.fake_search_return),
-            ])
-        self.adapter.register_uri(
+        self.register_uri(
+            'GET', 'https://image.example.com/v2/images',
+            json={'images': []})
+
+        self.register_uri(
             'POST', 'https://image.example.com/v2/tasks',
-            json=args)
-        self.adapter.register_uri(
-            'PATCH',
-            'https://image.example.com/v2/images/{id}'.format(
-                id=self.image_id))
-        self.adapter.register_uri(
-            'GET',
-            'https://image.example.com/v2/tasks/{id}'.format(id=task_id),
-            [
-                dict(status_code=503, text='Random error'),
-                dict(json={'images': args}),
-            ]
-        )
-
-        self.cloud.create_image(
-            image_name, self.imagefile.name, wait=True, timeout=1,
-            is_public=False, container=container_name)
-
-        args = {
-            'header': [
-                'x-object-meta-x-shade-md5:{md5}'.format(md5=NO_MD5),
-                'x-object-meta-x-shade-sha256:{sha}'.format(sha=NO_SHA256),
-            ],
-            'segment_size': 1000,
-            'use_slo': True}
-        swift_service_mock.upload.assert_called_with(
-            container='image_upload_v2_test_container',
-            objects=mock.ANY,
-            options=args)
-
-        self.calls += [
-            dict(method='GET', url='https://image.example.com/v2/images'),
-            dict(method='GET', url='https://object-store.example.com/info'),
-            dict(
-                method='HEAD',
-                url='{endpoint}/{container}'.format(
-                    endpoint=endpoint,
-                    container=container_name)),
-            dict(
-                method='PUT',
-                url='{endpoint}/{container}'.format(
-                    endpoint=endpoint,
-                    container=container_name)),
-            dict(
-                method='HEAD',
-                url='{endpoint}/{container}'.format(
-                    endpoint=endpoint,
-                    container=container_name)),
-            dict(
-                method='HEAD',
-                url='{endpoint}/{container}/{object}'.format(
-                    endpoint=endpoint,
-                    container=container_name, object=image_name)),
-            dict(method='GET', url='https://image.example.com/v2/images'),
-            dict(
-                method='POST',
-                url='https://image.example.com/v2/tasks',
+            json=args,
+            validate=dict(
                 json=dict(
                     type='import', input={
                         'import_from': '{container}/{object}'.format(
                             container=container_name, object=image_name),
-                        'image_properties': {'name': image_name}})),
-            dict(
-                method='GET',
-                url='https://image.example.com/v2/tasks/{id}'.format(
-                    id=task_id)),
-            dict(
-                method='GET',
-                url='https://image.example.com/v2/tasks/{id}'.format(
-                    id=task_id)),
-            dict(method='GET', url='https://image.example.com/v2/images'),
-            dict(
-                method='PATCH',
-                url='https://image.example.com/v2/images/{id}'.format(
-                    id=self.image_id),
+                        'image_properties': {'name': image_name}})))
+
+        self.register_uri(
+            'GET',
+            'https://image.example.com/v2/tasks/{id}'.format(id=task_id),
+            status_code=503, text='Random error')
+
+        self.register_uri(
+            'GET',
+            'https://image.example.com/v2/tasks/{id}'.format(id=task_id),
+            json={'images': args})
+
+        self.register_uri(
+            'GET', 'https://image.example.com/v2/images',
+            json={'images': [image_no_checksums]})
+
+        self.register_uri(
+            'PATCH',
+            'https://image.example.com/v2/images/{id}'.format(
+                id=self.image_id),
+            validate=dict(
                 json=sorted([
                     {
                         u'op': u'add',
@@ -372,9 +343,15 @@ class TestImage(base.RequestsMockTestCase):
                 headers={
                     'Content-Type':
                     'application/openstack-images-v2.1-json-patch'
-                }),
-            dict(method='GET', url='https://image.example.com/v2/images'),
-        ]
+                }))
+
+        self.register_uri(
+            'GET', 'https://image.example.com/v2/images',
+            json=self.fake_search_return)
+
+        self.cloud.create_image(
+            image_name, self.imagefile.name, wait=True, timeout=1,
+            is_public=False, container=container_name)
 
         self.assert_calls()
 
@@ -733,3 +710,19 @@ class TestImageV2Only(base.RequestsMockTestCase):
             self.cloud._image_client.get_endpoint())
         self.assertEqual(
             '2', self.cloud_config.get_api_version('image'))
+
+
+class TestImageVersionDiscovery(BaseTestImage):
+
+    def test_version_discovery_skip(self):
+        self.cloud.cloud_config.config['image_endpoint_override'] = \
+            'https://image.example.com/v2/override'
+
+        self.register_uri(
+            'GET', 'https://image.example.com/v2/override/images',
+            json={'images': []})
+        self.assertEqual([], self.cloud.list_images())
+        self.assertEqual(
+            self.cloud._image_client.endpoint_override,
+            'https://image.example.com/v2/override')
+        self.assert_calls()
